require 'draw_primitive.rb'
require 'otk_gl_panel.rb'

class MYGLPanelViewGerber < MyGLPanelBase
  def initialize(form_panel)
    super(form_panel)
    @triangle_object = []
    @line_object = []
    @outline_object = []
    @point_object = []
    @odd = false
    @min_x = 0
    @max_x = 0
    @min_y = 0
    @max_y = 0
    @eye_home_distance = 0
    @eye_home_x = 0
    @eye_home_y = 0
  end

  def clear_pattern_object()
    @triangle_object.clear()
    @line_object.clear()
    @min_x = 0
    @max_x = 0
    @min_y = 0
    @max_y = 0
  end

  def clear_outline_object
    @outline_object.clear()
  end

  def clear_drill_object
    @point_object.clear()
  end

  def view_reset
    d1 = @max_x - @min_x
    d2 = @max_y - @min_y
    if d1 > d2 then
      @eye_distance = d1 * 1.2
    else
      @eye_distance = d2 * 1.2
    end
    @eye_target_x = (@max_x + @min_x) / 2.0
    @eye_target_y = (@max_y + @min_y) / 2.0
    @eye_home_distance = @eye_distance
    @eye_home_x = @eye_target_x
    @eye_home_y = @eye_target_y
    refresh
  end

  def add_object( triangle )
    @triangle_object << triangle
    check_minmax(triangle.get_point(0))
    check_minmax(triangle.get_point(1))
    check_minmax(triangle.get_point(2))
  end

  def add_line( line )
    @line_object << line
    check_minmax(line[0])
    check_minmax(line[1])
  end

  def add_outline( line )
    @outline_object << line
    check_minmax(line[0])
    check_minmax(line[1])
  end

  def add_point( point )
    @point_object << point
    check_minmax(point)
  end

  def draw
    GL.Disable(EnableCap.DepthTest)
    # -- canvas
    GL.matrix_mode(MatrixMode.projection)
    GL.LoadIdentity()
    if @glview.width > @glview.height then
      screen_aspect = @glview.height.to_f / @glview.width.to_f
      GL.Ortho(-1.0, 1.0, screen_aspect, -screen_aspect, -1.0, 1.0)
      back_x = 1.0
      back_y = screen_aspect
    else
      screen_aspect = @glview.width.to_f / @glview.height.to_f
      GL.Ortho(-screen_aspect, screen_aspect, 1.0, -1.0, -1.0, 1.0)
      back_x = screen_aspect
      back_y = 1.0
    end
    GL.MatrixMode(MatrixMode.Modelview)
    GL.LoadIdentity()
    GL.Begin(BeginMode.Polygon)
    GL.Color3(0.0, 0.2, 0.4)
    GL.vertex2( back_x,  back_y)
    GL.Color3(0.0, 0.2, 0.4)
    GL.vertex2(-back_x,  back_y)
    GL.Color3(0.0, 0.0, 0.3)
    GL.vertex2(-back_x, -back_y)
    GL.Color3(0.0, 0.0, 0.3)
    GL.vertex2( back_x, -back_y)
    GL.end

    # -- parspective
    aspect_ratio = @glview.width.to_f / @glview.height.to_f
    perspective = OpenTK::Matrix4.create_perspective_field_of_view(
      MathHelper.PiOver4,
      aspect_ratio,
      0.1, 1000.0)
    GL.matrix_mode(MatrixMode.projection)
    GL.load_matrix(perspective)
    GL.MatrixMode(MatrixMode.Modelview);
    GL.load_matrix( cal_lookat() )
    # -- pattern
    @triangle_object.each do |obj|
      draw_gl_triangle(obj) if obj.ok?
    end
    @line_object.each do |obj|
      draw_gl_line(obj) if obj.ok?
    end
    @outline_object.each do |obj|
      draw_gl_line_2(obj) if obj.ok?
    end
    @point_object.each do |obj|
      draw_gl_point(obj)
    end
    GL.Enable(EnableCap.DepthTest)
  end

  def draw_gl_triangle(obj)
    p1 = obj.get_point(0)
    p2 = obj.get_point(1)
    p3 = obj.get_point(2)
#    GL.Color4(0.5, 1.0, 0.7, 0.5)
    GL.Color4(0.5, 1.0, 0.7, 1.0)
#    GL.Enable(EnableCap.Blend)
    GL.BlendFunc(BlendingFactorSrc.SrcAlpha, BlendingFactorDest.OneMinusSrcAlpha)
    GL.Begin(BeginMode.Triangles)
    GL.vertex3(p1.x, p1.y, p1.z)
    GL.vertex3(p2.x, p2.y, p2.z)
    GL.vertex3(p3.x, p3.y, p3.z)
    GL.end
    GL.Disable(EnableCap.Blend)
#    GL.Color3(Color.Cyan)
#    GL.Begin(BeginMode.LineLoop)
#    GL.vertex3(p1.x, p1.y, p1.z)
#    GL.vertex3(p2.x, p2.y, p2.z)
#    GL.vertex3(p3.x, p3.y, p3.z)
#    GL.end
  end

  def draw_gl_line(obj)
    p1 = obj.get_point(0)
    p2 = obj.get_point(1)
    GL.Color3(Color.Cyan)
    GL.Begin(BeginMode.Lines)
    GL.vertex3(p1.x, p1.y, p1.z)
    GL.vertex3(p2.x, p2.y, p2.z)
    GL.end
  end

  def draw_gl_line_2(obj)
    p1 = obj.get_point(0)
    p2 = obj.get_point(1)
    GL.Color3(Color.Orange)
    GL.Begin(BeginMode.Lines)
    GL.vertex3(p1.x, p1.y, p1.z)
    GL.vertex3(p2.x, p2.y, p2.z)
    GL.end
  end

  def draw_gl_point(obj)
    GL.Color3(Color.Red)
    GL.PointSize(2.0)
    GL.Begin(BeginMode.Points)
    GL.vertex3(obj.x, obj.y, obj.z)
    GL.end
  end

  # --------------------------------------
  # --- mouse operate
  # --------------------------------------

  def mouse_down(e)
    pos = @panel.PointToClient(System::Windows::Forms::Control.MousePosition)
    @mouse_pos_x = pos.X
    @mouse_pos_y = pos.Y
    if e.Button.to_s == 'Middle' then
      @eye_target_x = @eye_home_x
      @eye_target_y = @eye_home_y
      @eye_distance = @eye_home_distance
      refresh
    end
  end

  def mouse_up(e)
  end

  def mouse_move
    return if @mouse_drag == false
    pos = @panel.PointToClient(System::Windows::Forms::Control.MousePosition)
    mx = pos.X
    my = pos.Y
    if @glview.width > @glview.height then
      length = @glview.width
    else
      length = @glview.height
    end
    dx = (@mouse_pos_x - mx) / (length.to_f / @eye_distance)
    dy = (@mouse_pos_y - my) / (length.to_f / @eye_distance)
    @eye_target_x = @eye_target_x + dx
    @eye_target_y = @eye_target_y - dy
    @mouse_pos_x = mx
    @mouse_pos_y = my
    refresh
  end

  def mouse_change_wheel(e)
    wd = e.Delta / 120.0
    @eye_distance += (wd / 2.0)
    if @eye_distance < 0.5 then
      @eye_distance = 0.5
    end
    if @eye_distance > 1000.0 then
      @eye_distance = 1000.0
    end
    refresh
  end

  # ---------------------------------------------
  # --- model area

  def check_minmax(point)
    if @min_x > point.x then
      @min_x = point.x
    end
    if @max_x < point.x then
      @max_x = point.x
    end
    if @min_y > point.y then
      @min_y = point.y
    end
    if @max_y < point.y then
      @max_y = point.y
    end
  end

end
