#--------------------------------------------
#--- gerver and g-code file reader
#--------------------------------------------

class GFileReader

  def initialize
    @sequence = []
    @aperture = []
    @is_inch = false
    @scale = 1000
    # --- sequance holder
    @mode = 'off'
    @arc = 'off'
    @pos_x = 0.0
    @pos_y = 0.0
    @pos_i = 0.0
    @pos_j = 0.0
    @aprt = nil
  end

  attr_accessor :scale
  attr_accessor :is_inch

  def clear
    @sequence.clear
  end

  def get
    @sequence
  end

  def scale(mul)
    @scale
  end

  def load(filename)
    text = []
    extend = []
    open(filename, "r") do |fp|
      fp.each do |line|
        if line =~ /^%/ then
          extend << line
        elsif line =~ /^INCH,/
          @is_inch = true
        elsif line =~ /^METRIC,/
          @is_inch = false
        else
          text << line
        end
      end
    end
    if !parse_extend(extend) then
      puts "gerver extention error."
      return false
    end
    parse_gerber(text)
  end

  def parse_extend(text)
    text.each do |line|
      if line =~ /^%FS..X([0-9])([0-9])Y([0-9])([0-9])/ then
        @scale = 10 ** $2.to_i
      elsif line =~ /^%MOIN/ then
        @is_inch = true
      elsif line =~ /^%MOMM/ then
        @is_inch = false
      elsif line =~ /^%AD(D[0-9]+)([CRO]),(.*)\*/ then
        item = {}
        item['aperture'] = $1
        round = $2
        param = $3
        if round =~ /C/ then
          item['is_round'] = true
	  p = param.split('*')
          item['size'] = p[0].to_f
        elsif round =~ /R/
          item['is_round'] = false
	  p = param.split('X')
          item['size'] = p[0].to_f
          if p.size == 2 then
            item['size_height'] = p[1].to_f
          end
        elsif round =~ /O/
          item['is_round'] = true
	  p = param.split('X')
          item['size'] = p[0].to_f
          if p.size == 2 then
            item['size_height'] = p[1].to_f
          end
        end
        @aperture << item
      end
    end
  end

  def get_aperture(index)
    item = nil
    @aperture.each do |p|
      if p['aperture'] == index then
        item = p
        break
      end
    end
    return item
  end

  def parse_gerber(text)
    @mode = 'off'
    @arc = false
    @pos_x = 0.0
    @pos_y = 0.0
    @pos_i = 0.0
    @pos_j = 0.0
    @aprt = nil
    # -- parse
    text.each do |line|
      p = line.chop.split('*')
      p.each do |chunk|
       if chunk.size > 0 then
         parse_chunk(chunk)
       end
      end
    end
    return true
  end

  def parse_chunk(line)
    is_move = false
    tx = @pos_x
    ty = @pos_y
    ti = @pos_i
    tj = @pos_j
    # -- read
    if line =~ /D([0-9]+)/ then
      mode = $1.to_i
      if mode == 1 then
        @mode = 'on'
      elsif mode == 2 then
        @mode = 'off'
      elsif mode == 3 then
        @mode = 'flash'
        is_move = true
      else
        ap = sprintf("D%02d", mode)
        @aprt = get_aperture(ap)
      end
    end
    #
    if line =~ /G([0-9]+)/ then
      mode = $1.to_i
      if (mode == 0) || (mode == 1) then
        @arc = 'off'
      elsif (mode == 2) then
        @arc = '02'
      elsif (mode == 3) then
        @arc = '03'
      elsif (mode == 70) then
        @is_inch = true
      elsif (mode == 71) then
        @is_inch = false
      end
    end
    #
    if line =~ /X([-0-9]+)/ then
      tx = $1.to_f / @scale.to_f
      if @is_inch == true then
        tx = tx * 25.4
      end
      is_move = true
    end
    if line =~ /Y([-0-9]+)/ then
      ty = $1.to_f / @scale.to_f
      if @is_inch == true then
        ty = ty * 25.4
      end
      is_move = true
    end
    if line =~ /I([-0-9]+)/ then
      ti = $1.to_f / @scale.to_f
      if @is_inch == true then
        ti = ti * 25.4
      end
      is_move = true
    end
    if line =~ /J([-0-9]+)/ then
      tj = $1.to_f / @scale.to_f
      if @is_inch == true then
        tj = tj * 25.4
      end
      is_move = true
    end
    # -- output
    if is_move == true then
      if @mode == 'flash' then
        add_flash(tx,ty,tx,ty,@aprt) if @aprt != nil
        @mode = 'off'
      elsif @mode == 'on'
        if @arc == '02' then
          add_arc_1(@pos_x,@pos_y,tx,ty,ti,tj,@aprt) if @aprt != nil
        elsif @arc == '03' then
          add_arc_2(@pos_x,@pos_y,tx,ty,ti,tj,@aprt) if @aprt != nil
        else
          add_item(@pos_x,@pos_y,tx,ty,@aprt) if @aprt != nil
        end
      end
      @pos_x = tx
      @pos_y = ty
      @pos_i = ti
      @pos_j = tj
    end
  end

  def add_item(x1, y1, x2, y2, aperture)
    size = aperture['size']
    if @is_inch == true then
      size = size * 25.4
    end
    item = {}
    item['start_x'] = x1
    item['start_y'] = y1
    item['end_x'] = x2
    item['end_y'] = y2
    item['width'] = size
    item['is_round'] = aperture['is_round']
    @sequence << item
  end

  def add_flash(x1, y1, x2, y2, aperture)
    if aperture.key?('size_height') then
      acopy = aperture.dup
      if acopy['size'] > acopy['size_height'] then
        length = acopy['size'] - acopy['size_height']
        if @is_inch == true then
          length = length * 25.4
        end
        x1 -= length / 2.0
        x2 += length / 2.0
        acopy['size'] = acopy['size_height']
      else
        length = acopy['size_height'] - acopy['size']
        if @is_inch == true then
          length = length * 25.4
        end
        y1 -= length / 2.0
        y2 += length / 2.0
      end
      add_item(x1, y1, x2, y2, acopy)
    else
      add_item(x1, y1, x2, y2, aperture)
    end
  end

  def add_arc_1(x1, y1, x2, y2, i, j, aperture)
    cx = x1 + i
    cy = y1 + j
    v1 = PPoint.new(x1-cx, y1-cy)
    v2 = PPoint.new(x2-cx, y2-cy)
    vt = PPoint.new
    vt.copy(v1)
    r = Math.acos(v2.inner_product(v1)) * 360.0 / (Math::PI * 2)
    if v2.outer_product(v1).z < 0 then
      r = 360.0 - r
    end
    #
    while( r > 10.0 ) do
      vt.rotate_z(10.0)
      add_item(v1.x+cx, v1.y+cy, vt.x+cx, vt.y+cy, aperture)
      v1.copy(vt)
      r -= 10.0
    end
    add_item(v1.x+cx, v1.y+cy, v2.x+cx, v2.y+cy, aperture)
  end

  def add_arc_2(x1, y1, x2, y2, i, j, aperture)
    cx = x1 + i
    cy = y1 + j
    v1 = PPoint.new(x1-cx, y1-cy)
    v2 = PPoint.new(x2-cx, y2-cy)
    vt = PPoint.new
    vt.copy(v1)
    r = Math.acos(v2.inner_product(v1)) * 360.0 / (Math::PI * 2)
    if v2.outer_product(v1).z > 0 then
      r = 360.0 - r
    end
    #
    while( r > 10.0 ) do
      vt.rotate_z(-10.0)
      add_item(v1.x+cx, v1.y+cy, vt.x+cx, vt.y+cy, aperture)
      v1.copy(vt)
      r -= 10.0
    end
    add_item(v1.x+cx, v1.y+cy, v2.x+cx, v2.y+cy, aperture)
  end

end
