require "pattern_canvas.rb"

# --------------------------------------------------------
#  plot work
# --------------------------------------------------------

class PatternPlotter

  def initialize
    @mil_size = 0.0
    @pad_size = 0.5
    @flash_pattern = 0
    @pattern = []
    @circle_resolution = 12
    @marge_outline = true
  end

  attr_accessor :circle_resolution
  attr_accessor :marge_outline

  def clear_pattern
    @pattern.clear
  end

  def set_mil_size(size)
    @mil_size = size
  end

  def set_pad_size(size)
    @pad_size = size
  end

  def set_pad_round(type)
    if type == true then
      # round pattern
      @flash_pattern = 1
    else
      # square pattern
      @flash_pattern = 0
    end
  end

  def set_round_resolution(div)
    @circle_resolution = div
  end

  # -----------------------------
  # --- do plot

  def flash(x, y)
    item = PatternObject.new
    item.circle_resolution = @circle_resolution
    item.set_mil_size(@mil_size)
    item.set_pad_size(@pad_size)
    item.set_pad_round(@flash_pattern == 1)
    item.flash(x, y)
    item.drip_mil_path if @marge_outline == true
    add_item(item)
  end

  def move(x1, y1, x2, y2)
    item = PatternObject.new
    item.circle_resolution = @circle_resolution
    item.set_mil_size(@mil_size)
    item.set_pad_size(@pad_size)
    item.set_pad_round(@flash_pattern == 1)
    item.move(x1, y1, x2, y2)
    item.drip_mil_path if @marge_outline == true
    add_item(item)
  end

  def add_item(item)
    if @marge_outline == true then
      lines = item.get_mil_path
      deflines = []
      @pattern.each do |prim|
        if item.collision_bounding_box?(prim.bounding_box) then
          newlines = prim.marge_outline(lines, false)
          line2 = prim.get_mil_path
          newline2 = item.marge_outline(line2, true)
          prim.set_mil_path(newline2)
          item.set_mil_path(newlines)
          lines = newlines
        end
      end
    end
    @pattern << item
  end


  # -----------------------------
  # --- result

  def get_object
    newobj = []
    @pattern.each do |prim|
      prim.get_pattern_object.each do |tri|
        newobj << tri
      end
    end
    return newobj
  end

  def get_outline
    newobj = []
    @pattern.each do |prim|
      prim.get_mil_path.each do |line|
        newobj << line
      end
    end
    return newobj
  end

end
